/**
 * \file plotter_gnuplot.h
 *
 * \brief Header file to define the plotter gnuplot.This plotter
 * generate the source gnuplot code and the eps result file.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 *
 * \version 1.2
 *
 * \date 2015-03-21
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __PLOTTER_GNUPLOT_H__
#define __PLOTTER_GNUPLOT_H__

#include "plotters/plotter.h"
#include "tools/definition.h"
#include "tools/third_applications.h"

/**
 * \fn void plotter_gnuplot_pre_proc(Plotter *P);
 *
 * \brief From the plotter P, this function make the 
 * pre-processing for generate the Latex code for gnuplot figure.
 *
 * \param[in] P is a pointer to initialized plotter
 */
void plotter_gnuplot_pre_proc(Plotter *P);

/**
 * \fn void plotter_gnuplot_proc(Plotter *P);
 *
 * \brief From the plotter P, this function make the 
 * processing for generate the gnuplot figure.
 *
 * \param[in] P is a pointer to initialized plotter.
 */
void plotter_gnuplot_proc(Plotter *P);

/**
 * \fn void plotter_gnuplot_post_proc(Plotter *P);
 *
 * \brief From the plotter P, this function make the 
 * post-processing for generate the gnuplot picture.
 *
 * \param[in] P is a pointer to initialized plotter.
 */
void plotter_gnuplot_post_proc(Plotter *P);

/**
 * \fn void plotter_gnuplot_init(Plotter *P, const char* Plotter_filename, const char* output, const stegocurve_args *data);
 *
 * \brief This function initialize the plotter P with other arguments
 * for the gnuplot plotter.
 *
 * \param[in] P is a pointer to initialize.
 * \param[in] Plotter_filename is used to generated the gnuplot source.
 * \param[in] output is the name of the result file: that is the eps
 *            product file.
 * \param[in] data is the structure stegocurve_args which contains the
 *            paths to data files and their titles in the figure. 
 */
void plotter_gnuplot_init(Plotter *P, const char* Plotter_filename, const char* output, const stegocurve_args *data);

#endif /* __PLOTTER_GNUPLOT_H__  */
