/**
 *This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
 */

#include "codes/bch.h"

/*****************************************************************/
/*****************************************************************/
/*                               BCH2                            */
/*****************************************************************/
/*****************************************************************/
void init_BCH2_code(Code *C, const uint parameter)
{
  uint
    n = (uint) pow(2, parameter) - 1,
    k = n - 2*parameter;

  init_code_WC(C, n, k, 3);
}

void init_BCH2_codes(Code *C, const uint start, const uint nb)
{
  init_generic_codes(C, start, nb, init_BCH2_code);
}

void BCH2_codes_WC(const char* filename, const uint start, const uint nb)
{
  generic_codes_generic_cases(filename,
			      start,
			      nb,
			      init_BCH2_codes,
			      display_data_WC);
}


/*****************************************************************/
/*****************************************************************/
/*                            PBCH2                              */
/*****************************************************************/
/*****************************************************************/
void init_PBCH2_code(Code *C, const uint parameter)
{
  switch(parameter)
    {
    case 4:
      init_code_WC(C, 11, 7, 2);
      update_code_AC(C, 1.3750429700);
      break;

    case 5:
      init_code_WC(C, 28, 21, 2);
      update_code_AC(C, 1.7654476670);
      break;

    case 6:
      init_code_WC(C, 59, 51, 2);
      update_code_AC(C, 1.7773828038);
      break;

    case 7:
      init_code_WC(C, 123, 113, 2);
      update_code_AC(C, 1.8789928598);
      break;

    case 8:
      init_code_WC(C, 251, 239, 2);
      update_code_AC(C, 1.9389238972);
      break;

    case 9:
      init_code_WC(C, 507, 493, 2);
      update_code_AC(C, 1.9690576652);
      break;

    case 10:
      init_code_WC(C, 1018, 1003, 2);
      update_code_AC(C, 1.9692792437);
      break;

    case 11:
      init_code_WC(C, 2042, 2025, 2);
      update_code_AC(C, 1.9845902404);
      break;


    default:
      fprintf(stderr, "ERROR : init_PBCH2_code(parameter=%d)\nParameters not known!\n", parameter);      
      break;
    }
  /* Parameters find by experimentation on the first codes */
}

void init_PBCH2_codes(Code *C, const uint start, const uint nb)
{
  init_generic_codes(C, start, nb, init_PBCH2_code);
}

void PBCH2_codes_WC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      8,
			      init_PBCH2_codes,
			      display_data_WC);
}

void PBCH2_codes_AC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      8,
			      init_PBCH2_codes,
			      display_data_AC);
}




/*****************************************************************/
/*****************************************************************/
/*                             BCH3                              */
/*****************************************************************/
/*****************************************************************/
void init_BCH3_code(Code *C, const uint parameter)
{
  switch(parameter)
    {
    case 4:
      init_code_WC(C, 15, 5, 5);
      update_code_AC(C, 3.3333333333);
      break;
      
    case 5:
      init_code_WC(C, 31, 16, 5);
      update_code_AC(C, 4.2857142857);
      break;

    case 6:
      init_code_WC(C, 63, 45, 5);
      update_code_AC(C, 4.0632054176);
      break;
      
    case 7:
      init_code_WC(C, 127, 106, 5);
      update_code_AC(C, 3.8532110091);
      break;

    case 8:
      init_code_WC(C, 255, 231, 5);
      update_code_AC(C, 3.8400000000);
      break;

    case 9:
      init_code_WC(C, 511, 484, 5);
      update_code_AC(C, 3.8406827880);
      break;

    default:
      fprintf(stderr, "ERROR : init_BCH3_code(parameter=%d)\nParameters not known!\n", parameter);
      init_code_WC(C, 0, 0, 1);
      update_code_AC(C, 1);
      break;
    }
}

void init_BCH3_codes(Code *C, const uint start, const uint nb)
{
  init_generic_codes(C, start, nb, init_BCH3_code);
}

void BCH3_codes_WC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      6,
			      init_BCH3_codes,
			      display_data_WC);
}

void BCH3_codes_AC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      6,
			      init_BCH3_codes,
			      display_data_AC);
}





/*****************************************************************/
/*****************************************************************/
/*                             PBCH3                              */
/*****************************************************************/
/*****************************************************************/
void init_PBCH3_code(Code *C, const uint parameter)
{
  switch(parameter)
    {
    case 4:
      init_code_WC(C, 12, 5, 3);
      update_code_AC(C, 2.2950819672);
      break;
      
    case 5:
      init_code_WC(C, 25, 16, 3);
      update_code_AC(C, 2.4657534246);
      break;

    case 6:
      init_code_WC(C, 56, 45, 3);
      update_code_AC(C, 2.4336283185);
      break;
      
    case 7:
      init_code_WC(C, 121, 106, 3);
      update_code_AC(C, 2.7881040892);
      break;

    case 8:
      init_code_WC(C, 248, 231, 3);
      update_code_AC(C, 2.7823240589);
      break;

    case 9:
      init_code_WC(C, 504, 484, 3);
      update_code_AC(C, 2.8818443804);
      break;

    default:
      fprintf(stderr, "ERROR : init_PBCH3_code(parameter=%d)\nParameters not known!\n", parameter);
      init_code_WC(C, 0, 0, 1);
      update_code_AC(C, 1);
      break;
    }
}

void init_PBCH3_codes(Code *C, const uint start, const uint nb)
{
  init_generic_codes(C, start, nb, init_PBCH3_code);
}

void PBCH3_codes_WC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      6,
			      init_PBCH3_codes,
			      display_data_WC);
}

void PBCH3_codes_AC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      6,
			      init_PBCH3_codes,
			      display_data_AC);
}






/*****************************************************************/
/*****************************************************************/
/*                             BCH4                              */
/*****************************************************************/
/*****************************************************************/
void init_BCH4_code(Code *C, const uint parameter)
{
  switch(parameter)
    {
    case 4:
      init_code_WC(C, 15, 1, 7);
      update_code_AC(C, 5.92895507812500000000000000000);
      break;
      
    case 5:
      init_code_WC(C, 31, 11, 7);
      update_code_AC(C, 6.06852340698242187500000000000);
      break;

    case 6:
      init_code_WC(C, 63, 39, 7);
      update_code_AC(C, 5.59522318840026855468750000000);
      break;

    default:
      fprintf(stderr, "ERROR : init_BCH4_code(parameter=%d)\nParameters not known!\n", parameter);
      init_code_WC(C, 0, 0, 1);
      update_code_AC(C, 1);
      break;
    }
}

void init_BCH4_codes(Code *C, const uint start, const uint nb)
{
  init_generic_codes(C, start, nb, init_BCH4_code);
}

void BCH4_codes_WC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      3,
			      init_BCH4_codes,
			      display_data_WC);
}

void BCH4_codes_AC(const char* filename)
{
  generic_codes_generic_cases(filename,
			      4,
			      3,
			      init_BCH4_codes,
			      display_data_AC);
}
