#include "plotters/plotter_matplotlib.h"



void plotter_matplotlib_pre_proc(Plotter *P)
{
  fprintf(P->Plotter_file, "#!/usr/bin/python3\n");
  fprintf(P->Plotter_file, "# Automatically produced by stegocurve\n\n");

  fprintf(P->Plotter_file, "import matplotlib.pyplot as plt\n");
}

void plotter_matplotlib_proc(Plotter *P)
{
  int 
    size = stegocurve_args_get_size(P->data),
    i    = 1;

  fprintf(P->Plotter_file, "X, Y = [], []\n");
  fprintf(P->Plotter_file, "for line in open('%s', 'r'):\n", P->data->data[0][0]);
  fprintf(P->Plotter_file, "\t values = [float(s) for s in line.split()]\n");
  fprintf(P->Plotter_file, "\t X.append(values[0])\n");
  fprintf(P->Plotter_file, "\t Y.append(values[1])\n");
  fprintf(P->Plotter_file, "plt.plot(X, Y, color='red', linewidth='1', label='%s')\n\n", P->data->data[0][1]);

  for(; i<size; ++i)
    {
      fprintf(P->Plotter_file, "X, Y = [], []\n");
      fprintf(P->Plotter_file, "for line in open('%s', 'r'):\n", P->data->data[i][0]);
      fprintf(P->Plotter_file, "\t values = [float(s) for s in line.split()]\n");
      fprintf(P->Plotter_file, "\t X.append(values[0])\n");
      fprintf(P->Plotter_file, "\t Y.append(values[1])\n");
      fprintf(P->Plotter_file, "plt.plot(X, Y, '--.', label='%s')\n\n", P->data->data[i][1]);
    }
}

void plotter_matplotlib_post_proc(Plotter *P)
{
  fprintf(P->Plotter_file, "plt.xlim([0,1])\n");
  fprintf(P->Plotter_file, "plt.ylim([2,12])\n");
  fprintf(P->Plotter_file, "plt.legend(loc='upper right')\n");
  fprintf(P->Plotter_file, "plt.xlabel(r'$\\alpha$: relative payload')\n");
  fprintf(P->Plotter_file, "plt.ylabel('e: embedding efficiency')\n");
  fprintf(P->Plotter_file, "plt.title('Embedding efficiency as a function of relative payload')\n");
  fprintf(P->Plotter_file, "plt.savefig('%s')\n", P->output_filename);

  fclose(P->Plotter_file);
  P->Plotter_file = NULL;

#ifdef __linux__
  run_and_wait(2, MATPLOTLIB_COMMAND, MATPLOTLIB_FILE);
  run_and_wait(2, PDF_VIEWER, P->output_filename);
#endif /* __linux__ */
}


void plotter_matplotlib_init(Plotter *P, const char* Plotter_filename, const char* output, const stegocurve_args *data)
{
  char *output_filename;
  size_t size = strlen(output);

  if((output_filename = malloc((size + 5) * sizeof(char))) == NULL)
    {
      fprintf(stderr, "MEMORY ERROR: plotter_matplotlib_init() Impossible allocation.\n");
      return;
    }
  
  memcpy(output_filename, output, size);
  output_filename[size] = '\0';

  strncat(output_filename, ".pdf", 5);

  plotter_init(P,
	       plotter_matplotlib_pre_proc,
	       plotter_matplotlib_proc,
	       plotter_matplotlib_post_proc,
	       Plotter_filename,
	       output_filename,
	       data);

  free(output_filename);
}
