#include "plotters/plotter_tikz_pure.h"



void plotter_tikz_pure_pre_proc(Plotter *P)
{
  fprintf(P->Plotter_file,"\
\\documentclass{article}\n\n\
\\usepackage{tikz}\n\n\
\\title{Efficiency comparison for code based steganography}\n\
\\author{stegocurve}\n\n\
\\begin{document}\n\
\\maketitle\n\n");

  fprintf(P->Plotter_file,
	  "\\begin{figure}[ht]\n\
\\centering\n\
\\begin{tikzpicture}[x=10cm]\n\
    %% Styles definition:\n\
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n\
    %% for curves\n\n");

  fprintf(P->Plotter_file,
	  "\
    \\tikzstyle{bound}=[red, very thin, smooth];\n\
    \\tikzstyle{stegocurve1}=[blue, loosely dashed, very thick, mark=+];\n\
    \\tikzstyle{stegocurve2}=[green, loosely dotted, very thick, mark=*];\n\
    \\tikzstyle{stegocurve3}=[black, double, very thin];\n\
    \\tikzstyle{stegocurve4}=[brown, densely dashed, thin, mark=x];\n");
  fprintf(P->Plotter_file,
    "\
    \\tikzstyle{stegocurve5}=[violet, densely dotted, thin, mark=o];\n\
    \\tikzstyle{stegocurve6}=[orange, dashed, thick, mark=square];\n\
    \\tikzstyle{stegocurve7}=[magenta, dotted, thick, mark=oplus];\n\
    \\tikzstyle{stegocurve8}=[red, double, thick,];\n\
    \\tikzstyle{stegocurve9}=[blue, very thick, mark=|];\n\n");

  fprintf(P->Plotter_file,
	  "\
    %% for captions\n\
    \\tikzstyle{stegocurve1Title}=[stegocurve1, yshift=-0.5cm];\n\
    \\tikzstyle{stegocurve2Title}=[stegocurve2, yshift=-1cm];\n\
    \\tikzstyle{stegocurve3Title}=[stegocurve3, yshift=-1.5cm];\n\
    \\tikzstyle{stegocurve4Title}=[stegocurve4, yshift=-2cm];\n\
    \\tikzstyle{stegocurve5Title}=[stegocurve5, yshift=-2.5cm];\n");
  fprintf(P->Plotter_file,"\
    \\tikzstyle{stegocurve6Title}=[stegocurve6, yshift=-3cm];\n\
    \\tikzstyle{stegocurve7Title}=[stegocurve7, yshift=-3.5cm];\n\
    \\tikzstyle{stegocurve8Title}=[stegocurve8, yshift=-4cm];\n\
    \\tikzstyle{stegocurve9Title}=[stegocurve9, yshift=-4.5cm];\n\n");

  fprintf(P->Plotter_file,"\n\n\
    %% definition of the grid and x/y labels\n\
    \\draw[thin, dotted, gray!80] (0,2) grid[xstep=0.1] (1, 12);\n\
    \\foreach \\y in {2,4,...,12} \\draw(-0,\\y)node[left]{\\y};\n\
    \\foreach \\x in {0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1} \\draw(\\x,2)node[below]{\\x};\n\
    \\draw (0.5,1) node {$\\alpha$: relative payload};\n\
    \\draw (-0.1,6) node[rotate=90] {$e$: embedding efficiency};\n");
}

void plotter_tikz_pure_proc(Plotter *P)
{
  uint 
    size = stegocurve_args_get_size(P->data),
    i    = 1;

  fprintf(P->Plotter_file,"\
    %% print the captions\n\
    \\foreach \\style/\\title in {\n");

  for(; i<size; ++i)
    {
      fprintf(P->Plotter_file,"\
	      \tstegocurve%dTitle/%s,\n", i%10, P->data->data[i][1]);
    }
  fprintf(P->Plotter_file,"\
	  bound/%s\n",P->data->data[0][1]);
  fprintf(P->Plotter_file,"\
	  }\n\
             {\n\
               %% print the segment\n\
               \\draw[\\style] (0.5, 11.5)--++(0.05, 0) node[right] {\\title};\n\
               %% print the mark\n\
               \\draw[\\style] plot coordinates{(0.525,11.5)};\n\
             }\n\n");

  fprintf(P->Plotter_file,"\
      %% for the curves, cut the out of bound\n\
      \\clip (0,2) rectangle (1,12);\n\n\
    %% plot the curves\n\
    \\foreach \\style/\\file in {\n");
  
  fprintf(P->Plotter_file,"\
	  bound/{%s},\n", P->data->data[0][0]);
  for(i=1; i<size; ++i)
    {
      fprintf(P->Plotter_file,"\
	      stegocurve%d/{%s},\n", i, P->data->data[i][0]);
    }
  
  fprintf(P->Plotter_file,"\
	  }\n\
    \\draw[\\style] plot file {\\file};\n\
  \\end{tikzpicture}\n\
  \\caption{\\label{fig:stegocurve_tikz_pure}\n\
  Curve obtained by stegocurve and the plotter tikz\\_pure}\n\
\\end{figure}\n\
\\end{document}");

}

void plotter_tikz_pure_post_proc(Plotter *P)
{
  fclose(P->Plotter_file);
  P->Plotter_file = NULL;

#ifdef __linux__
  run_and_wait(2, LATEX_COMPILER, P->Plotter_filename);
  run_and_wait(2, PDF_VIEWER, P->output_filename);
#endif /* __linux__ */
}


void plotter_tikz_pure_init(Plotter *P, const char* Plotter_filename, const stegocurve_args *data)
{
  char 
    *tex_file,
    *pdf_file;

  size_t size = strlen(Plotter_filename);

  tex_file = malloc(5 + size * sizeof(char));
  pdf_file = malloc(5 + size * sizeof(char));

  memcpy(tex_file, Plotter_filename, size);
  memcpy(pdf_file, Plotter_filename, size);
  tex_file[size] = '\0';
  pdf_file[size] = '\0';

  strcat(tex_file, ".tex");
  strcat(pdf_file, ".pdf");

  plotter_init(P,
	       plotter_tikz_pure_pre_proc,
	       plotter_tikz_pure_proc,
	       plotter_tikz_pure_post_proc,
	       tex_file,
	       pdf_file,
	       data);

  free(pdf_file);
  free(tex_file);
}
