/**
 *This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
 */

#include "tools/entropy.h"


double q_log(const double x, const uint q)
{
  return(log(x)/log(q));
}


double entropy(const double x, const uint q)
{
  return(x*q_log(q-1,q) - x*q_log(x,q) - ((1-x) * q_log(1-x,q)));
}


void entropy_print_theoretical_bound(FILE *out, const double step, const uint q)
{
  double 
    i   = step,
    tmp;
  
  assert(out != NULL);

  for(; i<=((q-1)/(double)q); i+=step)
    {
      tmp = entropy(i,q);
      fprintf(out,"%f\t%f\n", tmp, tmp/i);
    }
}


void theoretical_bound(const char *filename, const double step, const uint q)
{
  FILE *out = fopen(filename, "w");

  assert(out != NULL);
  
  entropy_print_theoretical_bound(out, step, q);
  
  fclose(out);
}


