#include "tools/stegocurve_config.h"


void generate_all(const char *data_directory)
{
  uint
    start = 3,
    nb    = 20;

  char filename[100];

#ifdef __linux__
  mkdir(data_directory, 0744);
#endif /* __linux__ */

#ifdef __WIN32__
  mkdir(data_directory);
#endif /* __WIN32__ */

  sprintf(filename, "%s/entropy_2.dat", data_directory);
  theoretical_bound(filename, 0.0001, 2);

  sprintf(filename, "%s/Hamming_wc.dat", data_directory);
  Hamming_codes_WC(filename, start, nb);

  sprintf(filename, "%s/Hamming_ac.dat", data_directory);
  Hamming_codes_AC(filename, start, nb);


  sprintf(filename, "%s/RM_wc.dat", data_directory);
  RM_codes_WC(filename);

  sprintf(filename, "%s/BCH2_wc.dat", data_directory);
  BCH2_codes_WC (filename, start, nb);

  sprintf(filename, "%s/PBCH2_wc.dat", data_directory);
  PBCH2_codes_WC(filename);
  sprintf(filename, "%s/PBCH2_ac.dat", data_directory);
  PBCH2_codes_AC(filename);

  sprintf(filename, "%s/BCH3_wc.dat", data_directory);
  BCH3_codes_WC (filename);
  sprintf(filename, "%s/BCH3_ac.dat", data_directory);
  BCH3_codes_AC (filename);

  sprintf(filename, "%s/PBCH3_wc.dat", data_directory);
  PBCH3_codes_WC(filename);
  sprintf(filename, "%s/PBCH3_ac.dat", data_directory);
  PBCH3_codes_AC(filename);

  sprintf(filename, "%s/BCH4_wc.dat", data_directory);
  BCH4_codes_WC (filename);
  sprintf(filename, "%s/BCH4_ac.dat", data_directory);
  BCH4_codes_AC (filename);
}



void stegocurve_action_init(stegocurve_action *act)
{
  /* calloc() in the case where the action is generate, the other fields will */
  /* be initialized at zero value */
  if ((act->data = calloc(1, sizeof(stegocurve_args))) == NULL)
    {
      fprintf(stderr, "ERROR: stegocurve_action_init(). Memory unvailable.\n");
    }
}

void stegocurve_action_free(stegocurve_action *act)
{
  stegocurve_args_free(act->data);
  free(act->data);
  free(act->action_arg);
}


void stegocurve_action_print(const stegocurve_action *act)
{
  uint i = 0;

  if (act->action == error)
    {
      printf("ERROR: reading config file failed.\n");
      return;
    }

  printf("action:\t'%d'\n", act->action);
  printf("act arg:\t'%s'\n", act->action_arg);

  if (act->action != generate)
    {
      
      for(;i<stegocurve_args_get_size(act->data); ++i)
	{
	  printf("'%s'\t-->\t'%s'\n", 
		 stegocurve_args_get_datafile(act->data, i),
		 stegocurve_args_get_title(act->data, i));
	}
    }
  else
    {
      printf("PAS d'autre!\n");
    }
}



void stegocurve_action_do(const stegocurve_action *act)
{
  Plotter P;

  switch(act->action)
    {
    case generate:
      generate_all(act->action_arg);
      break;

    case gnuplot:
      printf("GNUPLOT Detected!\n");
      
      plotter_gnuplot_init(&P,
			    GNUPLOT_FILE,
			    act->action_arg,
			    act->data);

      plotter_run(&P);
      plotter_free(&P);
      break;

    case matplotlib:
      printf("MATPLOTLIB Detected!\n");
      
      plotter_matplotlib_init(&P,
			    MATPLOTLIB_FILE,
			    act->action_arg,
			    act->data);

      plotter_run(&P);
      plotter_free(&P);
      break;

    case pgfplots:
      printf("PGFPLOTS Detected!\n");

      plotter_pgfplots_init(&P,
			    act->action_arg,
			    act->data);

      plotter_run(&P);
      plotter_free(&P);
      break;

    case tikz:
      printf("TIKZ Detected!\n");

      plotter_tikz_pure_init(&P,
			    act->action_arg,
			    act->data);

      plotter_run(&P);
      plotter_free(&P);
      break;

    default:
      printf("ERROR: reading config file failed.\n");
      break;
    }
}





void extract_value_from_line(char **val, const char *line)
{
  char 
    *start,
    *end;

  uint size;

  start = strpbrk(line, "'\"");
  start++;

  end  = strchr(start, *(start-1));
  size = end - start;
  if((*val = malloc((size + 1) * sizeof(char))) == NULL)
    {
      fprintf(stderr, "ERROR: extract_value_from_line(). Memory unvailable.\n");
    }

  strncpy(*val, start, size);
  (*val)[size] = '\0';
}


void stegocurve_read_config_file(stegocurve_action *act, const char * config_filename)
{
  FILE 
    *f;
  
  char
    *val = NULL,
    line[LINE_LENGTH];

  uint
    file_ctr  = 0,
    title_ctr = 0;

  if((f = fopen(config_filename, "r")) == NULL)
    {
      fprintf(stderr, "ERROR: stegocurve_read_config_file(). %s cannot be open for reading.\n", config_filename);
      return;
    }

  /*
  printf("ouverture du fichier OK\n");
  */
  while(fgets(line, LINE_LENGTH, f))
    {

      /*      printf("traitement de la ligne:'%s'\n", line);
       */

      if ((line[0] != '#') && (line[0] != '\n'))
	{
	  if(! strncmp(line, "action", 6))
	    {
	      extract_value_from_line(&val, line);

	      if(! strncmp(val, "generate", 8))
		{
		  act->action = generate;
		}
	      else
	      if(! strncmp(val, "gnuplot", 7))
		{
		  act->action = gnuplot;
		}
	      else
	      if(! strncmp(val, "matplotlib", 9))
		{
		  act->action = matplotlib;
		}
	      else
	      if(! strncmp(val, "tikz", 4))
		{
		  act->action = tikz;
		}
	      else
	      if(! strncmp(val, "pgfplots", 8))
		{
		  act->action = pgfplots;
		}
	      else
		{
		  act->action = error;
		  printf("WARNING: Ignored line:\n%s\n", line);
		}

	      free(val);
	    }
	  else
	    if((! strncmp(line, "data_dir", 8)) && (act->action == generate))
	    {
	      extract_value_from_line(&(act->action_arg), line);
	      break;
	    }
	  else
	    if((! strncmp(line, "output_name", 11)) && (act->action != generate))
	    {
	      extract_value_from_line(&(act->action_arg), line);
	    }
	  else
	  if(! strncmp(line, "data_number", 11))
	    {
	      extract_value_from_line(&val, line);
	      stegocurve_args_init_size(act->data, strtol(val, NULL, 10));
	      free(val);
	    }
	  else
	  if(! strncmp(line, "data_file", 9))
	    {
	      if (file_ctr >= stegocurve_args_get_size(act->data))
		{
		  fprintf(stderr, "ERROR: stegocurve_read_config_file(). Too many data file.");
		}
	      else
		{
		  extract_value_from_line(&(act->data->data[file_ctr][0]), line);
		  file_ctr++;
		}
	    }
	  else
	  if(! strncmp(line, "data_title", 6))
	    {
	      if (title_ctr >= stegocurve_args_get_size(act->data))
		{
		  fprintf(stderr, "ERROR: stegocurve_read_config_file(). Too many title.");
		}
	      else
		{
		  extract_value_from_line(&(act->data->data[title_ctr][1]), line);
		  title_ctr++;
		}
	    }
	  else
	    {
	      printf("WARNING: Ignored line:\n%s\n",line);
	    }
	  
	}
    }

  fclose(f);
}


